@@ -118,37 +118,47 @@ static double S(double x,double y)
     }
 }
 
-static void gLasso(double *beta, double *x, double *w, double *r, int K0, int Kj, int n, double lambda, double *penpars, double lambda2, double *df)
+static void gLasso(double *beta, double *x, double *w, double *r, int K0, int Kj, int n, double lambda, double delta, double lambda2, double *df)
 {
   int i, j, k, j1, j2, k1, k2, K;
   K = Kj - K0;
-  double sxr, sxx, oldbeta, gradient_norm, sbb, ljk, delta, s;
+  double sxr, sxx, oldbeta, gradient_norm, sbb, ljk, s;
   double *u;
   u = vector(K);
-  delta = .00000001 + penpars[0]*lambda;
 
-  /* Calculate gradient_norm */
-  gradient_norm = 0;
-  for (j1=K0; j1<Kj; j1++)
+  for (j=K0; j<Kj; j++) sbb = sbb + pow(beta[j],2);
+  if (sbb==0)
     {
-      u[j1-K0] = 0;
-      for (i=0; i<n; i++)
+      gradient_norm = 0;
+      for (j1=K0; j1<Kj; j1++)
 	{
-	  u[j1-K0] = u[j1-K0] + x[n*j1+i]*w[i]*r[i];
-	  for (j2=K0; j2<Kj; j2++)
-	    {
-	      u[j1-K0] = u[j1-K0] + x[n*j1+i]*w[i]*x[n*j2+i]*beta[j2];
-	    }
+	  u[j1-K0] = 0;
+	  for (i=0; i<n; i++) u[j1-K0] = u[j1-K0] + x[n*j1+i]*w[i]*r[i];
+	  gradient_norm = gradient_norm + pow(u[j1-K0],2);
 	}
-      gradient_norm = gradient_norm + pow(u[j1-K0],2);
+      gradient_norm = sqrt(gradient_norm);
     }
-  gradient_norm = sqrt(gradient_norm);
-  /*gradient_norm = sqrt(gradient_norm);*/
-  /*Rprintf("%d %f %f \n",K0,gradient_norm/n,sqrt(K)*lambda);*/
-  if (gradient_norm/n > sqrt(K)*lambda + delta)
+  else
     {
-      sbb = delta;
-      for (j=K0; j<Kj; j++) sbb = sbb + pow(beta[j],2);
+      gradient_norm = 0;
+      for (j1=K0; j1<Kj; j1++)
+	{
+	  u[j1-K0] = 0;
+	  for (i=0; i<n; i++)
+	    {
+	      u[j1-K0] = u[j1-K0] + x[n*j1+i]*w[i]*r[i];
+	      for (j2=K0; j2<Kj; j2++)
+		{
+		  u[j1-K0] = u[j1-K0] + x[n*j1+i]*w[i]*x[n*j2+i]*beta[j2];
+		}
+	    }
+	  gradient_norm = gradient_norm + pow(u[j1-K0],2);
+	}
+      gradient_norm = sqrt(gradient_norm);
+    }
+  if (gradient_norm/n > sqrt(K)*lambda)
+    {
+      sbb = sbb + delta;
       for (j=K0; j<Kj; j++)
 	{
 	  oldbeta = beta[j];
@@ -162,36 +172,28 @@ static void gLasso(double *beta, double *x, double *w, double *r, int K0, int Kj
 	    }
 	  ljk = sqrt(K)*lambda/sqrt(sbb);
 	  beta[j] = (sxr+sxx*oldbeta)/(sxx+n*(ljk + lambda2));
-	  if (fabs(beta[j]-oldbeta) > .000001)
-	    {
-	      for (i=0; i<n; i++) r[i] = r[i] - (beta[j]-oldbeta)*x[n*j+i];
-	      sbb = sbb + pow(beta[j],2) - pow(oldbeta,2);
-	    }
-	  df[0] = df[0] + fabs(beta[j])/fabs(sxr/sxx+beta[j]);
+	  for (i=0; i<n; i++) r[i] = r[i] - (beta[j]-oldbeta)*x[n*j+i];
+	  sbb = sbb + pow(beta[j],2) - pow(oldbeta,2);
 	}
+      df[0] = df[0] + fabs(beta[j])/fabs(sxr/sxx+beta[j]);
     }
-  else
+  else if (beta[K0]!=0)
     {
       for (j=K0; j<Kj; j++)
 	{
-	  if (beta[j]==0) continue;
-	  else
-	    {
-	      oldbeta = beta[j];
-	      beta[j] = 0;
-	      for (i=0; i<n; i++) r[i] = r[i] + oldbeta*x[n*j+i];
-	    }
+	  oldbeta = beta[j];
+	  beta[j] = 0;
+	  for (i=0; i<n; i++) r[i] = r[i] + oldbeta*x[n*j+i];
 	}
     }
   free_vector(u);
 }
 
-static void gBridge(double *beta, double *x, double *w, double *r, int K0, int Kj, int n, double lambda, double *penpars, double lambda2, double *df)
+static void gBridge(double *beta, double *x, double *w, double *r, int K0, int Kj, int n, double lambda, double gamma, double lambda2, double *df)
 {
   int i, j, K;
   K = Kj - K0;
-  double sxr, sxx, oldbeta, sab, gamma, delta, ljk;
-  gamma = penpars[0];
+  double sxr, sxx, oldbeta, sab, delta, ljk;
   delta = .0000001;
 
   sab = 0;
@@ -215,7 +217,7 @@ static void gBridge(double *beta, double *x, double *w, double *r, int K0, int K
 	  beta[j] = S(sxr+sxx*oldbeta,n*ljk)/(sxx+n*lambda2);
 	}
       else beta[j] = 0;
-      if (fabs(beta[j]-oldbeta) > delta)
+      if (beta[j]!=oldbeta)
 	{
 	  for (i=0; i<n; i++) r[i] = r[i] - (beta[j]-oldbeta)*x[n*j+i];
 	  sab = sab + fabs(beta[j]) - fabs(oldbeta);
@@ -238,13 +240,12 @@ static double dMCP(double theta, double l, double a)
   else return(0);
 }
 
-static void gMCP(double *beta, double *x, double *w, double *r, int K0, int Kj, int n, double lambda, double lambda2, double *penpars, double *df)
+static void gMCP(double *beta, double *x, double *w, double *r, int K0, int Kj, int n, double lambda, double lambda2, double a, double *df)
 {
   int i, j, K;
   K = Kj - K0;
   double sxr, sxx, oldbeta, sMCP, ljk;
-  double phi,a,l1,l2,a1,a2;
-  a = penpars[0];
+  double phi,l1,l2,a1,a2;
   l1 = lambda;
   l2 = lambda;
   a1 = (K*a*pow(lambda,2))/(2*l1);
@@ -271,7 +272,7 @@ static void gMCP(double *beta, double *x, double *w, double *r, int K0, int Kj,
       /*Rprintf("%d %f %f %f %f %f %f\n",j,lambda,dMCP(sMCP,l1,a1),dMCP(beta[j],l2,a2),ljk,oldbeta,beta[j]);*/
       df[0] = df[0] + fabs(beta[j])/fabs(sxr/sxx+beta[j]);
 
-      if (fabs(beta[j]-oldbeta) > .00001)
+      if (beta[j]!=oldbeta)
 	{
 	  for (i=0; i<n; i++) r[i] = r[i] - (beta[j]-oldbeta)*x[j*n+i];
 	  sMCP = sMCP + MCP(beta[j],l2,a2) - MCP(oldbeta,l2,a2);
@@ -279,10 +280,10 @@ static void gMCP(double *beta, double *x, double *w, double *r, int K0, int Kj,
     }
 }
 
-static void gpPathFit(double *beta, int *counter, double *df, double *x, double *y, int *group, char **family, int *n, int *p, int *J, int *K, char **penalty, double *lambda, int *nlambda, double *eps, int *max_iter, int *verbose, int *monitor, int *n_monitor, double *penpars, double *lambda2, int *warn_conv)
+static void gpPathFit(double *beta, int *counter, double *df, double *x, double *y, int *group, char **family, int *n, int *p, int *J, int *K, char **penalty, double *lambda, int *nlambda, double *eps, int *max_iter, int *verbose, int *monitor, int *n_monitor, double *delta, double *gamma, double *a, double *lambda2, int *warn_conv)
 {
-  int l, i, j, g, K0, Kj, status, converged;
-  converged=0;
+  int l, i, j, g, K0, Kj, converged;
+  int saturated=0;
   double sxr, sxx, oldbeta;
   double ybar, yp, yy;
   double eta, pi;
@@ -378,7 +379,6 @@ static void gpPathFit(double *beta, int *counter, double *df, double *x, double
       while (counter[l] < *max_iter)
 	{
 	  converged = 0;
-	  status = 0;
 	  counter[l] = counter[l] + 1;
 	  if (*verbose) Rprintf("Iteration: %d\n",counter[l]);
 
@@ -410,6 +410,15 @@ static void gpPathFit(double *beta, int *counter, double *df, double *x, double
 		  yy = yy + pow(y[i]-ybar,2);
 		}
 	    }
+	  if (strcmp(family[0],"binomial")==0)
+	    {
+	      if (yp/yy < .01)
+		{
+		  warning("Model saturated; exiting...");
+		  saturated=1;
+		  break;
+		}
+	    }
 	  df[l] = 0;
 	  /*Rprintf("%f %f\n",r[1],X_[1][1]);*/
   
@@ -432,32 +441,24 @@ static void gpPathFit(double *beta, int *counter, double *df, double *x, double
 	      for (i=0; i<*n; i++) r[i] = r[i] - (Beta[l][j]-oldbeta)*x[j * *n + i];
 	      df[l] = df[l] + 1;
 	    }
+	  /* Update penalized covariates */
 	  for (g=0; g<*J; g++)
 	    {
 	      K0 = j;
 	      Kj = j + K[g];
 	      /*Rprintf("%d %d\n",K0,Kj);*/
 	      /*Rprintf("%f %f %f %f %f\n",beta[0],beta[1],beta[2],beta[3],beta[4]);*/
-	      if (strcmp(penalty[0],"gLasso")==0) gLasso(Beta[l],x,w,r,K0,Kj,*n,lambda[l],penpars,lambda2[l],&df[l]);
-	      if (strcmp(penalty[0],"gBridge")==0) gBridge(Beta[l],x,w,r,K0,Kj,*n,lambda[l],penpars,lambda2[l],&df[l]);
-	      if (strcmp(penalty[0],"gMCP")==0) gMCP(Beta[l],x,w,r,K0,Kj,*n,lambda[l],lambda2[l],penpars,&df[l]);
+	      if (strcmp(penalty[0],"gLasso")==0) gLasso(Beta[l],x,w,r,K0,Kj,*n,lambda[l],*delta,lambda2[l],&df[l]);
+	      if (strcmp(penalty[0],"gBridge")==0) gBridge(Beta[l],x,w,r,K0,Kj,*n,lambda[l],*gamma,lambda2[l],&df[l]);
+	      if (strcmp(penalty[0],"gMCP")==0) gMCP(Beta[l],x,w,r,K0,Kj,*n,lambda[l],lambda2[l],*a,&df[l]);
 	      j = Kj;
 	    }
-	  if (strcmp(family[0],"binomial")==0)
-	    {
-	      if (yp/yy < .01)
-		{
-		  warning("Model saturated; exiting...");
-		  status = 1;
-		}
-	    }
-	  if (status==1) break;
 	  /*Rprintf("%d %d\n",counter[0],max_iter[0]);
 	    Rprintf("%f %f %f\n",beta_new[0],beta_new[1],beta_new[2]);
 	    Rprintf("%f %f %f\n",beta_old[0],beta_old[1],beta_old[2]);*/
 	  if (*n_monitor != 0)
 	    {
-	      for (i=0; i<*n_monitor;i++) Rprintf("%.3f ",Beta[l][monitor[i]]);
+	      for (i=0; i<*n_monitor;i++) Rprintf("%.6f ",Beta[l][monitor[i]]);
 	      Rprintf("\n");
 	    }
 	  if (checkConvergence(Beta[l],beta_old,*eps,*p))
@@ -467,6 +468,7 @@ static void gpPathFit(double *beta, int *counter, double *df, double *x, double
 	    }
 	  for (j=0;j<*p;j++) beta_old[j] = Beta[l][j];
 	}
+      if (saturated) break;
       if (converged==0 & warn_conv[0]) warning("Failed to converge");
     }
   as_vector(beta,Beta,*nlambda,*p);
@@ -496,12 +498,11 @@ static void gLassoMax(double *max, double *x, double *r, double *w, int K0, int
   if (*max < max_g) *max = max_g;
 }
 
-static void gBridgeMax(double *max, double *x, double *r, double *w, int K0, int Kj, int n, double *penpars)
+static void gBridgeMax(double *max, double *x, double *r, double *w, int K0, int Kj, int n, double gamma)
 {
   int i, j, K;
   K = Kj - K0;
-  double sxr, max_g, gamma, delta;
-  gamma = penpars[0];
+  double sxr, max_g, delta;
   if (w[0]==1) delta = .35;
   else delta = .2;
 
@@ -514,12 +515,11 @@ static void gBridgeMax(double *max, double *x, double *r, double *w, int K0, int
     }
 }
 
-static void gMCPMax(double *max, double *x, double *r, double *w, int K0, int Kj, int n, double *penpars)
+static void gMCPMax(double *max, double *x, double *r, double *w, int K0, int Kj, int n, double a)
 {
   int i, j, K;
   K = Kj - K0;
-  double sxr, max_g, a, phi;
-  a = penpars[0];
+  double sxr, max_g, phi;
 
   for (j=K0; j<Kj; j++)
     {
@@ -530,7 +530,7 @@ static void gMCPMax(double *max, double *x, double *r, double *w, int K0, int Kj
     }
 }
 
-static void determineMax(double *max, double *x, double *r, double *w, int *group, char **family, int *n, int *p, int *J, int *K, char **penalty, double *penpars)
+static void determineMax(double *max, double *x, double *r, double *w, int *group, char **family, int *n, int *p, int *J, int *K, char **penalty, double *gamma, double *a)
 {
   int j, g, K0, Kj;
 
@@ -542,15 +542,15 @@ static void determineMax(double *max, double *x, double *r, double *w, int *grou
       /*Rprintf("%d %d\n",K0,Kj);*/
       /*Rprintf("%f %f %f %f %f\n",beta[0],beta[1],beta[2],beta[3],beta[4]);*/
       if (strcmp(penalty[0],"gLasso")==0) gLassoMax(max,x,r,w,K0,Kj,*n);
-      if (strcmp(penalty[0],"gBridge")==0) gBridgeMax(max,x,r,w,K0,Kj,*n,penpars);
-      if (strcmp(penalty[0],"gMCP")==0) gMCPMax(max,x,r,w,K0,Kj,*n,penpars);
+      if (strcmp(penalty[0],"gBridge")==0) gBridgeMax(max,x,r,w,K0,Kj,*n,*gamma);
+      if (strcmp(penalty[0],"gMCP")==0) gMCPMax(max,x,r,w,K0,Kj,*n,*a);
       j = Kj;
     }
 }
 
 static const R_CMethodDef cMethods[] = {
-  {"gpPathFit", (DL_FUNC) &gpPathFit, 22},
-  {"determineMax", (DL_FUNC) &determineMax, 12},
+  {"gpPathFit", (DL_FUNC) &gpPathFit, 24},
+  {"determineMax", (DL_FUNC) &determineMax, 13},
   NULL
 };
 
